package com.nsgk.agentcentersdk.api;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.nsgk.agentcentersdk.err.NSErrno;
import lombok.Data;

// 服务端返回对象
@Data
public final class NSApiResult
{
    public static final int NS_RESULT_SUCCESS = 200;
    public static final int NS_RESULT_ERROR = 500;

    public final int code;
    public final String msg;
    public final Object data;
    public final Long timestamp;
    public final int errno;

    NSApiResult(int code, String msg, Object data)
    {
        this(code, msg, data, NSErrno.ERRNO_OK);
    }

    NSApiResult(int code, String msg, Object data, int errno)
    {
        this(code, msg, data, System.currentTimeMillis(), errno);
    }

    NSApiResult(int code, String msg, Object data, Long timestamp, int errno)
    {
        this.code = code;
        this.msg = msg;
        this.data = data;
        this.timestamp = timestamp;
        this.errno = errno;
    }

    NSApiResult(boolean suc, String msg, Object data)
    {
        this(suc, msg, data, NSErrno.ERRNO_OK);
    }

    NSApiResult(boolean suc, String msg, Object data, int errno)
    {
        this(suc, msg, data, System.currentTimeMillis(), errno);
    }

    NSApiResult(boolean suc, String msg, Object data, Long timestamp, int errno)
    {
        this(suc ? NS_RESULT_SUCCESS : NS_RESULT_ERROR, msg, data, timestamp, errno);
    }

    public boolean IsSuccess()
    {
        return NS_RESULT_SUCCESS == code;
    }

    static NSApiResult Success(String msg, Object data)
    {
        return new NSApiResult(NS_RESULT_SUCCESS, msg, data);
    }

    static NSApiResult Error(int errno, String msg, Object data)
    {
        return new NSApiResult(NS_RESULT_ERROR, msg, data, errno);
    }

    static NSApiResult FromJSON(String str)
    {
        JSONObject jsonObject;

        jsonObject = JSON.parseObject(str);
        return new NSApiResult(
                jsonObject.getInteger("code"),
                jsonObject.getString("msg"),
                jsonObject.containsKey("data") ? jsonObject.get("data") : null,
                jsonObject.containsKey("timestamp") ? jsonObject.getLong("timestamp") : null,
                jsonObject.containsKey("errno") ? jsonObject.getInteger("errno") : null
        );
    }
}
