package com.nsgk.agentcentersdk.core;

import cn.hutool.core.bean.BeanUtil;
import com.alibaba.fastjson2.JSON;
import com.nsgk.agentcentersdk.err.NSErrGlobal;
import com.nsgk.agentcentersdk.err.NSErrno;
import com.nsgk.agentcentersdk.utility.NSSignTool;
import com.nsgk.agentcentersdk.utility.NSStr;
import lombok.Data;
import lombok.experimental.Accessors;

import java.util.Map;

// 封装对象
@Data
@Accessors(chain = true)
public class NSReportObject<T>
{
    protected static final String[] SIGN_FIELDS = { // 字母顺序
            "identifier", "protocol", "timestamp", "length",
    };
    private String identifier; // 识别ID
    private String sign; // 签名
    // md5(identifier=<客户端身份标识ID>&length=<data长度(加密前明文)>&protocol=<协议>&timestamp=<毫秒时间戳>)
    private int protocol = NSProtocol.NS_PROTOCOL_INVALID; // 协议
    private Long timestamp; // 客户端时间戳
    private T data;
    private String dataStr = "";

    public NSReportObject<T> setData(T d)
    {
        data = d;
        dataStr = "";
        if(null != data)
            dataStr = JSON.toJSONString(data);
        return this;
    }

    public NSReportObject<T> SetDataStr(String str, Class<T> clazz)
    {
        data = JSON.parseObject(str, clazz);
        dataStr = str;
        return this;
    }

    public NSReportObject()
    {
    }

    public NSReportObject(String identifier, int protocol)
    {
        this(identifier, protocol, null);
    }

    public NSReportObject(String identifier, int protocol, T data)
    {
        this.identifier = identifier;
        this.protocol = protocol;
        timestamp = System.currentTimeMillis();
        setData(data);
        sign = Sign();
    }

    public boolean IsValid()
    {
        return NSStr.IsNotEmpty(identifier) && null != timestamp && timestamp > 0 && NSProtocol.IsValid(protocol);
    }

    //"d10457fb8d1ae32b27f9eb9c0a107c75"
    public String Sign()
    {
        Map<String, Object> map;

        NSErrGlobal.ASSERT(IsValid(), NSErrno.ERRNO_CLI_PARAMETER_MISSING);
        try
        {
            map = BeanUtil.beanToMap(this);
            map.put("length", NSStr.Length(dataStr));
            return NSSignTool.Sign(map, SIGN_FIELDS);
        }
        catch(Exception e)
        {
            e.printStackTrace();
            return NSErrGlobal.ThrowAndReturnNull(NSErrno.ERRNO_SYS_INVALID_SIGN);
        }
    }

    public boolean CheckSign(String src)
    {
        if(!IsValid())
            return false;
        return src.equals(Sign());
    }

    public boolean CheckTime(long serverTime, long limit)
    {
        if(null == timestamp || timestamp <= 0L)
            return false;
        return serverTime - timestamp <= limit;
    }

    public NSReportObject<T> SetTimestampStr(String str)
    {
        timestamp = Long.parseLong(str);
        return this;
    }

    public NSReportObject<T> SetProtocolStr(String str)
    {
        protocol = NSProtocol.FromString(str);
        return this;
    }
}
