package com.nsgk.agentcentersdk.err;

import com.nsgk.agentcentersdk.utility.NSArr;

import java.io.Serializable;

// 全局错误记录
public final class NSErrGlobal
{
    private static final ThreadLocal<Integer> errno = new ThreadLocal<>();

    public static int Err(int newErr)
    {
        int err;

        err = errno.get();
        errno.set(newErr);
        return err;
    }

    public static void Throw(int err)
    {
        if(err == NSErrno.ERRNO_OK)
            return;
        throw new NSException(err);
    }

    public static void Throw()
    {
        Throw(Errno());
    }

    public static <T> T ThrowAndReturnNull(int err)
    {
        return ThrowAndReturn(err, null);
    }

    public static <T> T ThrowAndReturnNull()
    {
        return ThrowAndReturn(null);
    }

    public static <T> T ThrowAndReturn(int err, T t)
    {
        if(err == NSErrno.ERRNO_OK)
            return t;
        Err(err);
        throw new NSException(err);
    }

    public static <T> T ThrowAndReturn(T t)
    {
        return ThrowAndReturn(Errno(), t);
    }

    public static <T> T Return(int err, T t)
    {
        if(err != NSErrno.ERRNO_OK)
            Err(err);
        return t;
    }

    public static <T> T Return(T t)
    {
        return Return(Errno(), t);
    }

    // 默认清除
    public static int Errno(boolean...clear)
    {
        int err;

        err = errno.get();
        if(null == clear || clear.length == 0 || clear[0])
            errno.set(NSErrno.ERRNO_OK);
        return err;
    }

    public static boolean HasError(boolean...clear)
    {
        return Errno(clear) != NSErrno.ERRNO_OK;
    }

    public static String Error(boolean...clear)
    {
        return NSErrno.ErrStr(Errno(clear));
    }

    public static void ASSERT(boolean expression, Integer...err)
    {
        int e;

        e = NSArr.GetParm(NSErrno.ERRNO_ERROR, err);
        if(!expression)
            Throw(e);
    }

    private NSErrGlobal() {}
}
