package com.nsgk.agentcentersdk.utility;

import cn.hutool.core.codec.Base64;
import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.SecureUtil;
import cn.hutool.crypto.asymmetric.KeyType;
import cn.hutool.crypto.asymmetric.RSA;

import java.nio.charset.StandardCharsets;

// 加密工具
// 加密数据返回都为base64, 解密数据传参都为base64
// 客户端存放公钥, 服务端存放私钥
/*
        / ---------> \
    公钥加密            \      / -> 私钥解密
     /                  \   /              \
客户端                  服务端            处理/返回响应
     \                  /   \              /
   公钥解密             /      \ <- 私钥加密
       \ <---------  /
 */
public final class NSCrypto
{
    // RSA公钥加密(客户端) raw -> base64
    public static String RSAEncrypt_publicKey(String rawData, String publicKey_base64)
    {
        RSA rsa;

        if(NSStr.IsEmpty(rawData) || NSStr.IsEmpty(publicKey_base64))
            return "";
        rsa = SecureUtil.rsa(null, publicKey_base64);
        return rsa.encryptBase64(rawData, KeyType.PublicKey);
    }

    // RSA私钥解密(服务端) base64 -> raw
    public static String RSADecrypt_privateKey(String encryptBase64Data, String privateKey_base64)
    {
        RSA rsa;

        if(NSStr.IsEmpty(encryptBase64Data) || NSStr.IsEmpty(privateKey_base64))
            return "";
        rsa = SecureUtil.rsa(privateKey_base64, null);
        return StrUtil.str(rsa.decrypt(Base64.decode(encryptBase64Data), KeyType.PrivateKey), StandardCharsets.UTF_8);
    }

    // RSA私钥加密(服务端) raw -> base64
    public static String RSAEncrypt_privateKey(String rawData, String privateKey_base64)
    {
        RSA rsa;

        if(NSStr.IsEmpty(rawData) || NSStr.IsEmpty(privateKey_base64))
            return "";
        rsa = SecureUtil.rsa(privateKey_base64, null);
        return rsa.encryptBase64(rawData, KeyType.PrivateKey);
    }

    // RSA公钥解密(客户端) base64 -> raw
    public static String RSADecrypt_publicKey(String encryptBase64Data, String publicKey_base64)
    {
        RSA rsa;

        if(NSStr.IsEmpty(encryptBase64Data) || NSStr.IsEmpty(publicKey_base64))
            return "";
        rsa = SecureUtil.rsa(null, publicKey_base64);
        return StrUtil.str(rsa.decrypt(Base64.decode(encryptBase64Data), KeyType.PublicKey), StandardCharsets.UTF_8);
    }

    private NSCrypto() {}
}
