package com.nsgk.agentcentersdk.utility;

import java.lang.reflect.Field;
import java.lang.reflect.Method;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;
import java.util.Optional;

// 反射工具
public final class NSReflect
{
    public static Class<?> Class(String className)
    {
        try
        {
            return Class.forName(className);
        }
        catch(ClassNotFoundException e)
        {
            e.printStackTrace();
            throw new RuntimeException("SDK版本不匹配");
        }
    }

    @SuppressWarnings("unchecked")
    public static <T> T CallStaticT(Class<?> classObject, String methodName, Object...args)
    {
        return (T)CallStatic(classObject, methodName, args);
    }

    @SuppressWarnings("unchecked")
    public static <T> T CallStaticT_s(Class<?> classObject, String methodName, Object...args)
    {
        return (T)CallStatic_s(classObject, methodName, args);
    }

    public static Object CallStatic(Class<?> classObject, String methodName, Object...args)
    {
        if(null == args || args.length == 0)
            return CallStatic_s(classObject, methodName);
        Object[] objs = new Object[args.length * 2];
        for(int i = 0; i < args.length; i++)
        {
            objs[i * 2] = args[i].getClass();
            objs[i * 2 + 1] = args[i];
        }
        return CallStatic_s(classObject, methodName, objs);
    }

    public static Object CallStatic_s(Class<?> classObject, String methodName, Object...args)
    {
        List<Class<?>> classes = new ArrayList<>();
        List<Object> params = new ArrayList<>();
        if(null != args && args.length > 0)
        {
            for(int i = 0; i < args.length; i += 2)
            {
                Class<?> clazz;
                Object c = args[i];
                if(null != c)
                    clazz = (Class<?>) c;
                else
                    clazz = args[i + 1].getClass();
                classes.add(clazz);
                params.add(args[i + 1]);
            }
        }
        try
        {
            Method method = classObject.getMethod(methodName, classes.toArray(new Class<?>[0]));
            return method.invoke(null, params.toArray(new Object[0]));
        }
        catch(NoSuchMethodException | IllegalAccessException | IllegalArgumentException e)
        {
            e.printStackTrace();
            throw new RuntimeException("SDK版本不匹配");
        }
        catch(Exception e)
        {
            e.printStackTrace();
            throw new RuntimeException(e);
        }
    }

    @SuppressWarnings("unchecked")
    public static <T> T CallT(Object object, String methodName, Object...args)
    {
        return (T)Call(object, methodName, args);
    }

    @SuppressWarnings("unchecked")
    public static <T> T CallT_s(Object object, String methodName, Object...args)
    {
        return (T)Call_s(object, methodName, args);
    }

    public static Object Call(Object object, String methodName, Object...args)
    {
        if(null == args || args.length == 0)
            return Call_s(object, methodName);
        Object[] objs = new Object[args.length * 2];
        for(int i = 0; i < args.length; i++)
        {
            objs[i * 2] = args[i].getClass();
            objs[i * 2 + 1] = args[i];
        }
        return Call_s(object, methodName, objs);
    }

    public static Object Call_s(Object object, String methodName, Object...args)
    {
        List<Class<?>> classes = new ArrayList<>();
        List<Object> params = new ArrayList<>();
        if(null != args && args.length > 0)
        {
            for(int i = 0; i < args.length; i += 2)
            {
                Class<?> clazz;
                Object c = args[i];
                if(null != c)
                    clazz = (Class<?>) c;
                else
                    clazz = args[i + 1].getClass();
                classes.add(clazz);
                params.add(args[i + 1]);
            }
        }
        try
        {
            Method method = object.getClass().getMethod(methodName, classes.toArray(new Class<?>[0]));
            return method.invoke(object, params.toArray(new Object[0]));
        }
        catch(NoSuchMethodException | IllegalAccessException | IllegalArgumentException e)
        {
            e.printStackTrace();
            throw new RuntimeException("SDK版本不匹配");
        }
        catch(Exception e)
        {
            e.printStackTrace();
            throw new RuntimeException(e);
        }
    }

    // find first: 不检查类型
    public static Field FindField(Class<?> clazz, String name)
    {
        Field[] declaredFields;
        Optional<Field> first;
        Class<?> superclass;

        declaredFields = clazz.getDeclaredFields();
        first = Arrays.stream(declaredFields).filter((x) -> x.getName().equals(name)).findFirst();
        if(first.isPresent())
            return first.get();

        superclass = clazz.getSuperclass();
        if(null != superclass)
            return FindField(superclass, name);
        return null;
    }

    // find first: 检查类型, 仅匹配
    public static Field FindField_s(Class<?> clazz, String name, Class<?> fieldClazz)
    {
        Field[] declaredFields;
        Optional<Field> first;
        Class<?> superclass;

        declaredFields = clazz.getDeclaredFields();
        first = Arrays.stream(declaredFields).filter((x) -> x.getName().equals(name) && x.getType().isAssignableFrom(fieldClazz)).findFirst();
        if(first.isPresent())
            return first.get();

        superclass = clazz.getSuperclass();
        if(null != superclass)
            return FindField(superclass, name);
        return null;
    }

    // find first: 检查类型, 严格
    public static Field FindField_ss(Class<?> clazz, String name, Class<?> fieldClazz)
    {
        Field[] declaredFields;
        Optional<Field> first;
        Class<?> superclass;

        declaredFields = clazz.getDeclaredFields();
        first = Arrays.stream(declaredFields).filter((x) -> x.getName().equals(name) && x.getType().equals(fieldClazz)).findFirst();
        if(first.isPresent())
            return first.get();

        superclass = clazz.getSuperclass();
        if(null != superclass)
            return FindField(superclass, name);
        return null;
    }

    private NSReflect() {}
}
