package com.ruoyi.web.controller.agentcenter;

import com.ruoyi.agentcenter.dto.AgentTaskNumGroup;
import com.ruoyi.agentcenter.vo.TaskAllAudit;
import com.ruoyi.agentcenter.vo.TaskAudit;
import com.ruoyi.agentcenter.vo.TaskDistrib;
import com.ruoyi.agentcenter.vo.TaskRevoke;
import com.ruoyi.common.core.domain.AjaxResult;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.agentcenter.domain.TAgentTask;
import com.ruoyi.agentcenter.service.ITAgentTaskService;
import com.ruoyi.common.core.page.TableDataInfo;

import java.util.List;

/**
 * 主管任务Controller
 * 
 * @author zhao
 * @date 2023-05-06
 */
@RestController
@RequestMapping("/manager/task")
public class ManagerAgentTaskController extends BaseController
{
    @Autowired
    private ITAgentTaskService tAgentTaskService;

    /**
     * 县级全部任务统计
     */
    @PreAuthorize("@ss.hasPermi('agentcenter:task:list')")
    @GetMapping(value = "/countyTaskSummary")
    public AjaxResult countyTaskSummary(TAgentTask task)
    {
        task.setAgentCenter(getUserAgentCenter());
        task.setTownCode(null);
        return AjaxResult.success(tAgentTaskService.countyTaskSummary(task));
    }

    /**
     * 全部任务列表
     */
    @PreAuthorize("@ss.hasPermi('agentcenter:task:list')")
    @GetMapping(value = "/allTaskList")
    public TableDataInfo allTaskList(TAgentTask task)
    {
        startPage();
        task.setAgentCenter(getUserAgentCenter());
        return getDataTable(tAgentTaskService.getTownTaskList(task));
    }

    /**
     * 待分配任务列表
     */
    @PreAuthorize("@ss.hasPermi('agentcenter:task:list')")
    @GetMapping(value = "/undistribTaskList")
    public AjaxResult undistribTaskList(TAgentTask task)
    {
        startPage();
        task.setAgentCenter(getUserAgentCenter());
        task.setAgentStatus(TAgentTask.AGENT_STATUS_READY);
        AgentTaskNumGroup summary = new AgentTaskNumGroup();
        List<AgentTaskNumGroup> undistribTaskList = tAgentTaskService.getUndistribTaskList(task, summary);
        TableDataInfo dataTable = getDataTable(undistribTaskList);
        AjaxResult result = dataTable.toAjaxResult();
        result.put("numVillage", summary.getNumVillage())
                .put("numBook", summary.getNumBook())
                .put("allCount", summary.getAllCount())
                ;
        return result;
    }

    /**
     * 分配
     */
    @PreAuthorize("@ss.hasPermi('manager:task:distrib')")
    @PostMapping(value = "/distrib")
    public AjaxResult distrib(@Validated @RequestBody TaskDistrib taskDistrib)
    {
        taskDistrib.setDistriUser(getUsername());
        taskDistrib.setAgentCenter(getUserAgentCenter());
        return toAjax(tAgentTaskService.distrib(taskDistrib));
    }

    /**
     * 撤回
     */
    @PreAuthorize("@ss.hasPermi('manager:task:revoke')")
    @PostMapping(value = "/revoke")
    public AjaxResult revoke(@Validated @RequestBody TaskRevoke taskRevoke)
    {
        taskRevoke.setAgentCenter(getUserAgentCenter());
        return toAjax(tAgentTaskService.revoke(taskRevoke));
    }

    /**
     * 审核
     */
    @PreAuthorize("@ss.hasPermi('manager:task:audit')")
    @PostMapping(value = "/audit")
    public AjaxResult audit(@Validated @RequestBody TaskAudit taskAudit)
    {
        taskAudit.setAuditUser(getUsername());
        taskAudit.setAgentCenter(getUserAgentCenter());
        return toAjax(tAgentTaskService.audit(taskAudit));
    }

    /**
     * 全部审核
     */
    @PreAuthorize("@ss.hasPermi('manager:task:audit')")
    @PostMapping(value = "/auditAll")
    public AjaxResult auditAll(@Validated @RequestBody TaskAllAudit taskAudit)
    {
        taskAudit.setAuditUser(getUsername());
        taskAudit.setAgentCenter(getUserAgentCenter());
        return toAjax(tAgentTaskService.auditAll(taskAudit));
    }

    /**
     * 处理中任务列表
     */
    @PreAuthorize("@ss.hasPermi('agentcenter:task:list')")
    @GetMapping(value = "/processingTaskList")
    public TableDataInfo processingTaskList(TAgentTask task)
    {
        startPage();
        task.setAgentCenter(getUserAgentCenter());
        return getDataTable(tAgentTaskService.getProcessingTaskList(task));
    }

    /**
     * 县级处理中统计
     */
    @PreAuthorize("@ss.hasPermi('agentcenter:task:list')")
    @GetMapping(value = "/countyProcessingTaskSummary")
    public AjaxResult countyProcessingTaskSummary(TAgentTask task)
    {
        task.setAgentCenter(getUserAgentCenter());
        task.setTownCode(null);
        return AjaxResult.success(tAgentTaskService.countyProcessingTaskSummary(task));
    }

    /**
     * 待审核任务列表
     */
    @PreAuthorize("@ss.hasPermi('agentcenter:task:list')")
    @GetMapping(value = "/auditingTaskList")
    public TableDataInfo auditingTaskList(TAgentTask task)
    {
        startPage();
        task.setAgentCenter(getUserAgentCenter());
        return getDataTable(tAgentTaskService.getAuditingTaskList(task));
    }

    /**
     * 镇级待审核统计
     */
    @PreAuthorize("@ss.hasPermi('agentcenter:task:list')")
    @GetMapping(value = "/auditTaskSummary")
    public AjaxResult auditTaskSummary(TAgentTask task)
    {
        task.setAgentCenter(getUserAgentCenter());
        return AjaxResult.success(tAgentTaskService.auditTaskSummary(task));
    }

}
