package com.ruoyi.agentcenter.listener;

import cn.hutool.core.lang.Assert;
import com.nsgk.agentcentersdk.core.NSProtocol;
import com.nsgk.agentcentersdk.message.NSContractionMessage;
import com.ruoyi.agentcenter.domain.TAgentContraction;
import com.ruoyi.agentcenter.domain.TAgentTask;
import com.ruoyi.agentcenter.object.Session;
import com.ruoyi.agentcenter.service.ITAgentContractionService;
import com.ruoyi.agentcenter.service.ITAgentTaskService;
import com.ruoyi.common.utils.handler.CallbackHandler;
import com.ruoyi.common.utils.handler.HandlerEngine;
import com.ruoyi.common.utils.sql.SqlUtil;
import com.ruoyi.system.service.ISysDeptService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import javax.annotation.PostConstruct;
import java.util.Date;

@Component
public class ContractionListener
{
    @Autowired
    private ITAgentContractionService itAgentContractionService;
    @Autowired
    private ITAgentTaskService itAgentTaskService;
    @Autowired
    private ISysDeptService    iSysDeptService;

    @PostConstruct
    public void init()
    {
        HandlerEngine.Register(this);
    }

    @CallbackHandler(protocol = "" + NSProtocol.NS_PROTOCOL_CONTRACTION)
    public void reportContraction(Session<NSContractionMessage> session)
    {
        TAgentContraction contraction = conv(session.data);
        String orgCode = contraction.getOrgCode();

        SqlUtil.transaction(() -> {
            if(null == contraction.getId())
            {
                TAgentTask tAgentTask = itAgentTaskService.getTAgentTask(orgCode, contraction.getOrderYear(), contraction.getOrderMonth(), (x) -> {
                    x.setBookId(contraction.getBookId())
                     .setBookName(session.data.getBookName())
                    ;
                });

                TAgentTask task = new TAgentTask();
                task.setContracCount(1);
                task.setId(tAgentTask.getId());
                itAgentTaskService.updateTAgentTaskCount(task);

                contraction.setTaskId(tAgentTask.getId());
                itAgentContractionService.insertTAgentContraction(contraction);
                session.result.setMessage("新增合同报账成功");
            }
            else
            {
                itAgentContractionService.updateTAgentContraction(contraction);
                session.result.setMessage("更新合同报账成功");
            }
        });

        session.result.setData(contraction);
    }

    @CallbackHandler(protocol = "" + NSProtocol.NS_PROTOCOL_UPDATE_CONTRACTION)
    public void updateContraction(Session<NSContractionMessage> session)
    {
        NSContractionMessage inContraction = session.data;
        TAgentContraction contraction = itAgentContractionService.selectTAgentContraction(new TAgentContraction().setOrgCode(inContraction.getOrgCode()).setOutId(inContraction.getOutId()));
        Assert.notNull(contraction, "报账合同不存在");
        Assert.isTrue(TAgentTask.AGENT_STATUS_PROCESSING.equals(contraction.getAgentStatus()), "报账合同任务未分配");
        Assert.isTrue("2".equals(contraction.getOrderStatus()), "报账合同状态不是待记账状态");

        SqlUtil.transaction(() -> {
            contraction.setOrderStatus(inContraction.getOrderStatus());
            itAgentContractionService.updateTAgentContraction(contraction);
            syncFinishTask(contraction);
        });

        session.result.setData(contraction);
    }

    private TAgentContraction conv(NSContractionMessage data)
    {
        TAgentContraction contractionCond = new TAgentContraction();
        contractionCond.setOrgCode(data.getOrgCode())
                .setOutId(data.getOutId());
        TAgentContraction contraction = itAgentContractionService.selectTAgentContraction(contractionCond);
        if(null != contraction)
        {
            Assert.isTrue("2".equals(contraction.getOrderStatus()), "非待记账状态的合同保障不能修改");
            contraction.setBookId(data.getBookId())
                       .setDeptId(data.getDeptId())
                       .setName(data.getName())
                       .setBuildingTime(data.getBuildingTime())
                       ;
            contraction.setUpdateBy("admin");
        }
        else
        {
            contraction = new TAgentContraction();
            contraction.setBookId(data.getBookId())
                       .setDeptId(data.getDeptId())
                       .setOutId(data.getOutId())
                       .setName(data.getName())
                       .setOrgCode(data.getOrgCode())
                       .setBuildingTime(data.getBuildingTime())
                       .setOrderAt(new Date())
                       .setAgentStatus(TAgentTask.AGENT_STATUS_READY)
                       .setOrderStatus("2")
                       .normalized()
            ;
            contraction.setCreateBy("admin");
        }

        return contraction;
    }

    private void syncFinishTask(TAgentContraction contraction)
    {
        String orgCode = contraction.getOrgCode();

        TAgentTask task = new TAgentTask();
        task.setOrgCode(orgCode)
            .setOrderYear(contraction.getOrderYear())
            .setOrderMonth(contraction.getOrderMonth())
            ;
        task.setFinishCount(1);
        itAgentTaskService.syncTAgentTaskCount(task);
    }
}
