package com.ruoyi.agentcenter.service;

import java.util.List;
import java.util.function.Consumer;

import com.ruoyi.agentcenter.domain.TAgentTask;
import com.ruoyi.agentcenter.dto.AgentTaskTownGroup;
import com.ruoyi.agentcenter.dto.AgentTaskNumGroup;
import com.ruoyi.agentcenter.dto.AgentTaskVillageGroup;
import com.ruoyi.agentcenter.vo.AgentTaskUser;
import com.ruoyi.agentcenter.vo.AgentTaskSummary;
import com.ruoyi.agentcenter.vo.TaskAllAudit;
import com.ruoyi.agentcenter.vo.TaskAudit;
import com.ruoyi.agentcenter.vo.TaskDistrib;
import com.ruoyi.agentcenter.vo.TaskFinish;
import com.ruoyi.agentcenter.vo.TaskRevoke;

/**
 * 任务清单Service接口
 * 
 * @author zhao
 * @date 2023-07-14
 */
public interface ITAgentTaskService 
{
    /**
     * 查询任务清单
     * 
     * @param id 任务清单主键
     * @return 任务清单
     */
    public TAgentTask selectTAgentTaskById(Long id);

    /**
     * 查询任务清单列表
     * 
     * @param tAgentTask 任务清单
     * @return 任务清单集合
     */
    public List<TAgentTask> selectTAgentTaskList(TAgentTask tAgentTask);

    /**
     * 新增任务清单
     * 
     * @param tAgentTask 任务清单
     * @return 结果
     */
    public int insertTAgentTask(TAgentTask tAgentTask);

    /**
     * 批量新增任务清单
     *
     * @param list 任务清单
     * @return 结果
     */
    public int insertTAgentTaskBatch(List<TAgentTask> list);

    /**
     * 修改任务清单
     * 
     * @param tAgentTask 任务清单
     * @return 结果
     */
    public int updateTAgentTask(TAgentTask tAgentTask);

    /**
     * 批量修改 任务清单
     *
     * @param list 任务清单
     * @return 结果
     */
    public int updateTAgentTaskBatch(List<TAgentTask> list);

    /**
     * 批量删除任务清单
     * 
     * @param ids 需要删除的任务清单主键集合
     * @return 结果
     */
    public int deleteTAgentTaskByIds(Long[] ids);

    /**
     * 删除任务清单信息
     * 
     * @param id 任务清单主键
     * @return 结果
     */
    public int deleteTAgentTaskById(Long id);
    
    // Harm
    /**
     * 条件单条查询任务清单
     *
     * @param tAgentTask 任务清单
     * @return 任务清单条目
     */
    public TAgentTask selectTAgentTask(TAgentTask tAgentTask);

    /**
     * 条件查询任务清单数量
     *
     * @param tAgentTask 任务清单
     * @return 任务清单数量
     */
    public Long selectTAgentTaskCount(TAgentTask tAgentTask);

    /**
     * 条件查询任务清单是否存在
     *
     * @param tAgentTask 任务清单
     * @return 任务清单是否存在
     */
    public boolean selectTAgentTaskExists(TAgentTask tAgentTask);

    public int updateTAgentTaskCount(TAgentTask tAgentTask);

    public void fillTAgentTaskOrg(TAgentTask tAgentTask);

    public TAgentTask getTAgentTask(String orgCode, String year, String month, Consumer<TAgentTask> ifNotExists);

    public TAgentTask getTAgentTask(String orgCode, String year, String month);

    public int syncTAgentTaskCount(TAgentTask tAgentTask);

    public List<AgentTaskTownGroup> getTownTaskList(TAgentTask tAgentTask);
    public List<AgentTaskTownGroup> getProcessingTaskList(TAgentTask tAgentTask);
    public List<AgentTaskNumGroup> getAuditingTaskList(TAgentTask tAgentTask);

    public List<TAgentTask> getTodoTaskList(TAgentTask tAgentTask);
    public List<AgentTaskVillageGroup> getProcessedTaskList(TAgentTask tAgentTask);
    public Long getAgentTaskOverdueFinishVillageCount(TAgentTask tAgentTask);

    public int distrib(TaskDistrib task);
    public int revoke(TaskRevoke task);
    public int audit(TaskAudit task);
    public int finish(TaskFinish task);
    public int auditAll(TaskAllAudit task);
    public List<AgentTaskUser> userList(AgentTaskUser user);

    public List<AgentTaskNumGroup> getUndistribTaskList(TAgentTask tAgentTask, AgentTaskNumGroup summary);
    public AgentTaskSummary countyTaskSummary(TAgentTask task);
    public AgentTaskSummary countyProcessingTaskSummary(TAgentTask task);
    public AgentTaskSummary auditTaskSummary(TAgentTask task);
}
