package com.ruoyi.agentcenter.service.impl;

import cn.hutool.core.lang.Assert;
import com.nsgk.agentcentersdk.api.NSSDKServer;
import com.nsgk.agentcentersdk.core.NSProtocol;
import com.nsgk.agentcentersdk.core.NSReportObject;
import com.nsgk.agentcentersdk.message.NSContractionMessage;
import com.nsgk.agentcentersdk.message.NSMessage;
import com.nsgk.agentcentersdk.err.NSErrGlobal;
import com.nsgk.agentcentersdk.err.NSErrno;
import com.nsgk.agentcentersdk.err.NSException;
import com.ruoyi.agentcenter.object.Session;
import com.ruoyi.agentcenter.service.IAgentCenter;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.utils.handler.HandlerEngine;
import com.ruoyi.system.mapper.SysDeptMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;

@Service
public class AgentCenterImpl implements IAgentCenter
{
    @Autowired
    private SysDeptMapper sysDeptMapper;

    public int getProtocol(HttpServletRequest request)
    {
        int protocol = NSSDKServer.GetProtocol(request);
        if(!NSProtocol.IsValid(protocol))
        {
            NSErrGlobal.Err(NSErrno.ERRNO_SYS_INVALID_PROTOCOL);
            throw new NSException(NSErrno.ERRNO_SYS_INVALID_PROTOCOL, String.format("未知的协议类型: 0x%x", protocol));
        }
        return protocol;
    }

    public Session<? extends NSMessage> createSession(NSSDKServer server, HttpServletRequest request)
    {
        NSReportObject<? extends NSMessage> reportObject;
        Session<? extends NSMessage> session;
        Class<? extends NSMessage> clazz;
        int protocol;

        protocol = getProtocol(request);
        switch(protocol)
        {
            case NSProtocol.NS_PROTOCOL_CONTRACTION:
            case NSProtocol.NS_PROTOCOL_UPDATE_CONTRACTION:
                clazz = NSContractionMessage.class;
                break;
            default:
                return null;
        }
        reportObject = server.Recv(request, clazz);
        if(!reportObject.IsValid() || !reportObject.getData().IsValid())
        {
            return NSErrGlobal.ThrowAndReturn(NSErrno.ERRNO_CLI_DATA_MISSING, null);
        }

        session = new Session<>(protocol, reportObject.getIdentifier(), reportObject.getTimestamp(), reportObject.getData());

        return session;
    }

    private void check(Session<? extends NSMessage> session, HttpServletRequest request)
    {
        Assert.notNull(session, "无效消息");
        String identifier = session.identifier;
        Assert.notBlank(identifier, "无效客户端标识");
        SysDept dept = sysDeptMapper.selectSysDeptByOrgCode(identifier);
        Assert.notNull(dept, "不支持的地区: {}", identifier);
        String remoteUrl = dept.getRemoteUrl();
        Assert.notBlank(remoteUrl, "该地区无项目: {}", identifier);

        session.data.setDeptId(dept.getDeptId());
        /*try
        {
            URL url = new URL(remoteUrl);
            String host = url.getHost();
            String clientHost = request.getRemoteAddr();
            Assert.isTrue(clientHost.equals(host), "该客户端部署在不支持的地址: {}", clientHost);

            int port = url.getPort();
            int clientPort = request.getRemotePort();
            Assert.isTrue(clientPort == port, "该客户端部署在不支持的地址: {}", clientPort);
        }
        catch(Exception e)
        {
            e.printStackTrace();
            throw new RuntimeException(e);
        }*/
    }

    @Override
    public Session<?> postHandle(NSSDKServer server, HttpServletRequest request)
    {
        Session<? extends NSMessage> session;

        session = createSession(server, request);
        check(session, request);
        HandlerEngine.Call("" + session.protocol, session);
        return session;
    }
}
