package com.ruoyi.agentcenter.service.impl;

import java.util.List;
import com.ruoyi.common.utils.DateUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.apache.commons.collections4.ListUtils;
import com.ruoyi.agentcenter.mapper.TAgentContractionMapper;
import com.ruoyi.agentcenter.domain.TAgentContraction;
import com.ruoyi.agentcenter.service.ITAgentContractionService;

/**
 * 合同任务Service业务层处理
 * 
 * @author zhao
 * @date 2023-07-14
 */
@Service
public class TAgentContractionServiceImpl implements ITAgentContractionService 
{
    @Autowired
    private TAgentContractionMapper tAgentContractionMapper;

    /**
     * 查询合同任务
     * 
     * @param id 合同任务主键
     * @return 合同任务
     */
    @Override
    public TAgentContraction selectTAgentContractionById(Long id)
    {
        return tAgentContractionMapper.selectTAgentContractionById(id);
    }

    /**
     * 查询合同任务列表
     * 
     * @param tAgentContraction 合同任务
     * @return 合同任务
     */
    @Override
    public List<TAgentContraction> selectTAgentContractionList(TAgentContraction tAgentContraction)
    {
        return tAgentContractionMapper.selectTAgentContractionList(tAgentContraction);
    }

    /**
     * 新增合同任务
     * 
     * @param tAgentContraction 合同任务
     * @return 结果
     */
    @Override
    public int insertTAgentContraction(TAgentContraction tAgentContraction)
    {
        tAgentContraction.setCreateTime(DateUtils.getNowDate());
        return tAgentContractionMapper.insertTAgentContraction(tAgentContraction);
    }

    /**
     * 批量新增合同任务
     *
     * @param list 合同任务
     * @return 结果
     */
    @Override
    @Transactional
    public int insertTAgentContractionBatch(List<TAgentContraction> list){
        List<List<TAgentContraction>> splists = ListUtils.partition(list, 50);
        splists.forEach(splist->{
            tAgentContractionMapper.insertTAgentContractionBatch(splist);
        });
        return 1;
    }

    /**
     * 修改合同任务
     * 
     * @param tAgentContraction 合同任务
     * @return 结果
     */
    @Override
    public int updateTAgentContraction(TAgentContraction tAgentContraction)
    {
        tAgentContraction.setUpdateTime(DateUtils.getNowDate());
        return tAgentContractionMapper.updateTAgentContraction(tAgentContraction);
    }

    /**
     * 批量修改 合同任务
     *
     * @param list 合同任务
     * @return 结果
     */
    @Override
    @Transactional
    public int updateTAgentContractionBatch(List<TAgentContraction> list) {
        List<List<TAgentContraction>> splists = ListUtils.partition(list, 50);
        splists.forEach(splist->{
            tAgentContractionMapper.updateTAgentContractionBatch(splist);
        });
        return 1;
    }

    /**
     * 批量删除合同任务
     * 
     * @param ids 需要删除的合同任务主键
     * @return 结果
     */
    @Override
    public int deleteTAgentContractionByIds(Long[] ids)
    {
        return tAgentContractionMapper.deleteTAgentContractionByIds(ids);
    }

    /**
     * 删除合同任务信息
     * 
     * @param id 合同任务主键
     * @return 结果
     */
    @Override
    public int deleteTAgentContractionById(Long id)
    {
        return tAgentContractionMapper.deleteTAgentContractionById(id);
    }

    // Harm
    /**
     * 单条条件查询合同任务
     *
     * @param tAgentContraction 合同任务
     * @return 合同任务条目
     */
    @Override
    public TAgentContraction selectTAgentContraction(TAgentContraction tAgentContraction) {
        return tAgentContractionMapper.selectTAgentContraction(tAgentContraction);
    }

    /**
     * 条件查询合同任务数量
     *
     * @param tAgentContraction 合同任务
     * @return 合同任务数量
     */
    @Override
    public Long selectTAgentContractionCount(TAgentContraction tAgentContraction) {
        return tAgentContractionMapper.selectTAgentContractionCount(tAgentContraction);
    }

    /**
     * 条件查询合同任务是否存在
     *
     * @param tAgentContraction 合同任务
     * @return 合同任务是否存在
     */
    @Override
    public boolean selectTAgentContractionExists(TAgentContraction tAgentContraction) {
        return tAgentContractionMapper.selectTAgentContractionExists(tAgentContraction) > 0;
    }
}
