package com.ruoyi.agentcenter.service.impl;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.function.Consumer;

import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.lang.Assert;
import com.github.pagehelper.Page;
import com.github.pagehelper.PageInfo;
import com.ruoyi.agentcenter.dto.AgentTaskTownGroup;
import com.ruoyi.agentcenter.dto.AgentTaskNumGroup;
import com.ruoyi.agentcenter.dto.AgentTaskVillageGroup;
import com.ruoyi.agentcenter.vo.AgentTaskUser;
import com.ruoyi.agentcenter.vo.AgentTaskSummary;
import com.ruoyi.agentcenter.vo.TaskAllAudit;
import com.ruoyi.agentcenter.vo.TaskAudit;
import com.ruoyi.agentcenter.vo.TaskDistrib;
import com.ruoyi.agentcenter.vo.TaskFinish;
import com.ruoyi.agentcenter.vo.TaskRevoke;
import com.ruoyi.common.core.domain.entity.SysDept;
import com.ruoyi.common.core.domain.entity.SysUser;
import com.ruoyi.common.exception.ASSERT;
import com.ruoyi.common.utils.ContainerUtils;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.DeptUtils;
import com.ruoyi.common.utils.PageUtils;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.system.mapper.SysDeptMapper;
import com.ruoyi.system.mapper.SysUserMapper;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.apache.commons.collections4.ListUtils;
import com.ruoyi.agentcenter.mapper.TAgentTaskMapper;
import com.ruoyi.agentcenter.domain.TAgentTask;
import com.ruoyi.agentcenter.service.ITAgentTaskService;

/**
 * 任务清单Service业务层处理
 * 
 * @author zhao
 * @date 2023-07-14
 */
@Service
public class TAgentTaskServiceImpl implements ITAgentTaskService 
{
    @Autowired
    private TAgentTaskMapper tAgentTaskMapper;
    @Autowired
    private SysDeptMapper    sysDeptMapper;
    @Autowired
    private SysUserMapper    sysUserMapper;

    /**
     * 查询任务清单
     * 
     * @param id 任务清单主键
     * @return 任务清单
     */
    @Override
    public TAgentTask selectTAgentTaskById(Long id)
    {
        return tAgentTaskMapper.selectTAgentTaskById(id);
    }

    /**
     * 查询任务清单列表
     * 
     * @param tAgentTask 任务清单
     * @return 任务清单
     */
    @Override
    public List<TAgentTask> selectTAgentTaskList(TAgentTask tAgentTask)
    {
        return tAgentTaskMapper.selectTAgentTaskList(tAgentTask);
    }

    /**
     * 新增任务清单
     * 
     * @param tAgentTask 任务清单
     * @return 结果
     */
    @Override
    public int insertTAgentTask(TAgentTask tAgentTask)
    {
        tAgentTask.setCreateTime(DateUtils.getNowDate());
        return tAgentTaskMapper.insertTAgentTask(tAgentTask);
    }

    /**
     * 批量新增任务清单
     *
     * @param list 任务清单
     * @return 结果
     */
    @Override
    @Transactional
    public int insertTAgentTaskBatch(List<TAgentTask> list){
        List<List<TAgentTask>> splists = ListUtils.partition(list, 50);
        splists.forEach(splist->{
            tAgentTaskMapper.insertTAgentTaskBatch(splist);
        });
        return 1;
    }

    /**
     * 修改任务清单
     * 
     * @param tAgentTask 任务清单
     * @return 结果
     */
    @Override
    public int updateTAgentTask(TAgentTask tAgentTask)
    {
        tAgentTask.setUpdateTime(DateUtils.getNowDate());
        return tAgentTaskMapper.updateTAgentTask(tAgentTask);
    }

    /**
     * 批量修改 任务清单
     *
     * @param list 任务清单
     * @return 结果
     */
    @Override
    @Transactional
    public int updateTAgentTaskBatch(List<TAgentTask> list) {
        List<List<TAgentTask>> splists = ListUtils.partition(list, 50);
        splists.forEach(splist->{
            tAgentTaskMapper.updateTAgentTaskBatch(splist);
        });
        return 1;
    }

    /**
     * 批量删除任务清单
     * 
     * @param ids 需要删除的任务清单主键
     * @return 结果
     */
    @Override
    public int deleteTAgentTaskByIds(Long[] ids)
    {
        return tAgentTaskMapper.deleteTAgentTaskByIds(ids);
    }

    /**
     * 删除任务清单信息
     * 
     * @param id 任务清单主键
     * @return 结果
     */
    @Override
    public int deleteTAgentTaskById(Long id)
    {
        return tAgentTaskMapper.deleteTAgentTaskById(id);
    }

    // Harm
    /**
     * 单条条件查询任务清单
     *
     * @param tAgentTask 任务清单
     * @return 任务清单条目
     */
    @Override
    public TAgentTask selectTAgentTask(TAgentTask tAgentTask) {
        return tAgentTaskMapper.selectTAgentTask(tAgentTask);
    }

    /**
     * 条件查询任务清单数量
     *
     * @param tAgentTask 任务清单
     * @return 任务清单数量
     */
    @Override
    public Long selectTAgentTaskCount(TAgentTask tAgentTask) {
        return tAgentTaskMapper.selectTAgentTaskCount(tAgentTask);
    }

    /**
     * 条件查询任务清单是否存在
     *
     * @param tAgentTask 任务清单
     * @return 任务清单是否存在
     */
    @Override
    public boolean selectTAgentTaskExists(TAgentTask tAgentTask) {
        return tAgentTaskMapper.selectTAgentTaskExists(tAgentTask) > 0;
    }

    @Override
    public int updateTAgentTaskCount(TAgentTask tAgentTask)
    {
        return tAgentTaskMapper.updateTAgentTaskCount(tAgentTask);
    }

    @Override
    public void fillTAgentTaskOrg(TAgentTask tAgentTask)
    {
        String orgCode = tAgentTask.getOrgCode();
        int level = DeptUtils.parseOrgCodeLevel(orgCode);
        Assert.isTrue(level >= DeptUtils.LEVEL_TOWN, "orgCode必须是镇级或镇以下级别");

        SysDept deptCond = new SysDept();
        deptCond.setOrgCode(orgCode);
        SysDept org = sysDeptMapper.selectSysDept(deptCond);
        Assert.notNull(org, "orgCode对应的地区不存在: {} - {}", orgCode, level);
        List<Long> levelIds = org.makeAncestorsIdList(true);

        SysDept town;
        SysDept county;

        if(level > DeptUtils.LEVEL_TOWN) // 镇级以下
        {
            town = sysDeptMapper.selectDeptById(levelIds.get(DeptUtils.LEVEL_TOWN));
            Assert.notNull(town, "orgCode对应的镇级地区不存在: {} - {}", orgCode, level);
        }
        else
            town = org;

        county = sysDeptMapper.selectDeptById(levelIds.get(DeptUtils.LEVEL_COUNTY));
        Assert.notNull(county, "orgCode对应的区级地区不存在: {} - {}", orgCode, level);

        tAgentTask.setCountyCode(county.getOrgCode())
                .setCountyName(county.getDeptName())
                .setTownCode(town.getOrgCode())
                .setTownName(town.getDeptName())
                .setOrgName(org.getDeptName())
                  ;
    }

    @Override
    public TAgentTask getTAgentTask(String orgCode, String year, String month, Consumer<TAgentTask> ifNotExists)
    {
        TAgentTask tAgentTask = getTAgentTask(orgCode, year, month);
        if(null == tAgentTask)
        {
            tAgentTask = new TAgentTask();
            SysDept deptCond = new SysDept();
            deptCond.setOrgCode(orgCode);
            SysDept dept = sysDeptMapper.selectSysDept(deptCond);
            Assert.notNull(dept, "部门不存在: {}", orgCode);
            tAgentTask.setOrgCode(orgCode)
                .setOrderYear(year)
                .setOrderMonth(month)

                .setContracCount(0)
                .setVoucherCount(0)
                .setAssetCount(0)
                .setAllCount(0)
                .setFinishCount(0)

                .setAgentCenter(dept.getAgentCenter())
                .setAgentStatus(TAgentTask.AGENT_STATUS_READY)
            ;
            fillTAgentTaskOrg(tAgentTask);
            tAgentTask.setCreateBy("admin");
            ifNotExists.accept(tAgentTask); // 补充
            insertTAgentTask(tAgentTask);
        }
        return tAgentTask;
    }

    @Override
    public TAgentTask getTAgentTask(String orgCode, String year, String month)
    {
        Assert.notBlank(orgCode, "查询任务: orgCode为空");
        Assert.notBlank(year, "查询任务: orderYear为空");
        Assert.notBlank(month, "查询任务: orderMonth为空");

        TAgentTask task = new TAgentTask();
        task.setOrgCode(orgCode)
            .setOrderYear(year)
            .setOrderMonth(month)
        ;
        return tAgentTaskMapper.selectTAgentTask(task);
    }

    @Override
    public int syncTAgentTaskCount(TAgentTask queryTask)
    {
        TAgentTask dbTask = getTAgentTask(queryTask.getOrgCode(), queryTask.getOrderYear(), queryTask.getOrderMonth());
        Assert.notNull(dbTask, "{}下{}年{}月无任务", queryTask.getOrgCode(), queryTask.getOrderYear(), queryTask.getOrderMonth());

        TAgentTask updateTask = new TAgentTask();
        updateTask.setId(dbTask.getId());
        updateTask.setFinishCount(queryTask.getFinishCount())
                .setContracCount(queryTask.getContracCount())
                .setAssetCount(queryTask.getAssetCount())
                .setVoucherCount(queryTask.getVoucherCount())
        ;
        return updateTAgentTaskCount(updateTask);
    }

    @Override
    public List<AgentTaskTownGroup> getTownTaskList(TAgentTask tAgentTask)
    {
        List<AgentTaskTownGroup> agentTaskGroupByTown = tAgentTaskMapper.getAgentTaskGroupByTown(tAgentTask);
        if(CollectionUtil.isNotEmpty(agentTaskGroupByTown))
        {
            List<String> townCodeList = ContainerUtils.mapToList(agentTaskGroupByTown, AgentTaskTownGroup::getTownCode);

            List<AgentTaskVillageGroup> villageGroups = tAgentTaskMapper.getAgentTaskGroupByVillage(new TAgentTask().putParam("townCodeList", townCodeList));
            Map<String, List<AgentTaskVillageGroup>> villageMap = ContainerUtils.groupingBy(villageGroups, AgentTaskVillageGroup::getTownCode);

            List<String> orgCodeList = ContainerUtils.mapToList(villageGroups, AgentTaskVillageGroup::getOrgCode);

            TAgentTask agentTaskCond = new TAgentTask();
            agentTaskCond.putParam("orgCodeList", orgCodeList);
            List<TAgentTask> tAgentTasks = tAgentTaskMapper.selectTAgentTaskList(agentTaskCond);
            Map<String, List<TAgentTask>> bookGroup = ContainerUtils.groupingBy(tAgentTasks, TAgentTask::getOrgCode);

            agentTaskGroupByTown.forEach((town) -> {
                List<AgentTaskVillageGroup> villageList = villageMap.getOrDefault(town.getTownCode(), new ArrayList<>());
                long numApprovalFinish = villageList.stream().filter((x) -> x.getNum().equals(x.getNumApprovalFinish())).count();
                long numProcessFinish = villageList.stream().filter((x) -> x.getNum().equals(x.getNumProcessFinish())).count();
                town.setNumApprovalFinish(numApprovalFinish);
                town.setNumProcessFinish(numProcessFinish);
                town.setVillageList(villageList);
                villageList.forEach((village) -> {
                    village.setApprovalFinish(village.getNum().equals(village.getNumApprovalFinish()));
                    village.setDistriDate(DateUtils.stdRemoveTime(village.getDistriDate()));
                    village.setHandleDate(DateUtils.stdRemoveTime(village.getHandleDate()));
                    village.setTaskList(bookGroup.getOrDefault(village.getOrgCode(), new ArrayList<>()));
                    village.getTaskList().forEach((x) -> {
                        x.setOtherCount(x.getAllCount() - x.getVoucherCount() - x.getContracCount());
                    });
                });
            });
        }
        return agentTaskGroupByTown;
    }

    @Override
    public List<TAgentTask> getTodoTaskList(TAgentTask tAgentTask)
    {
        tAgentTask.setAgentStatus(TAgentTask.AGENT_STATUS_PROCESSING);
        List<TAgentTask> tAgentTasks = tAgentTaskMapper.selectTAgentTaskList(tAgentTask);
        tAgentTasks.forEach((x) -> {
            x.setOtherCount(x.getAllCount() - x.getContracCount() - x.getVoucherCount());
        });
        return tAgentTasks;
    }

    @Override
    public List<AgentTaskVillageGroup> getProcessedTaskList(TAgentTask tAgentTask)
    {
        tAgentTask.putParam("agentStatusList", Arrays.asList(TAgentTask.AGENT_STATUS_FINISHED, TAgentTask.AGENT_STATUS_AUDITED));
        List<AgentTaskVillageGroup> list = tAgentTaskMapper.getProcessedAgentTaskGroupByVillage(tAgentTask);
        if(CollectionUtil.isNotEmpty(list))
        {
            PageUtils.orderBy("org_code");
            List<TAgentTask> tAgentTasks = tAgentTaskMapper.getTAgentTaskProcessedList(tAgentTask);
            tAgentTasks.forEach((x) -> {
                x.setOtherCount(x.getAllCount() - x.getContracCount() - x.getVoucherCount());
            });
            Map<String, List<TAgentTask>> taskGroup = ContainerUtils.groupingBy(tAgentTasks, TAgentTask::getOrgCode);
            list.forEach((x) -> {
                x.setDistriDate(DateUtils.stdRemoveTime(x.getDistriDate()));
                x.setHandleDate(DateUtils.stdRemoveTime(x.getHandleDate()));
                x.setTaskList(taskGroup.getOrDefault(x.getOrgCode(), new ArrayList<>()));
            });
        }
        return list;
    }

    @Override
    public Long getAgentTaskOverdueFinishVillageCount(TAgentTask tAgentTask)
    {
        tAgentTask.putParam("agentStatusList", Arrays.asList(TAgentTask.AGENT_STATUS_FINISHED, TAgentTask.AGENT_STATUS_AUDITED));
        return tAgentTaskMapper.getAgentTaskOverdueFinishVillageCount(tAgentTask);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public int distrib(TaskDistrib task)
    {
        List<String> orgCodeList = ContainerUtils.mapToList(task.getItems(), TaskDistrib.TaskDistribItem::getOrgCode);
        TAgentTask cond = new TAgentTask();
        cond.setOrderYear(task.getOrderYear())
                .setOrderMonth(task.getOrderMonth())
                .setAgentCenter(task.getAgentCenter())
                .setAgentStatus(TAgentTask.AGENT_STATUS_READY)
                .putParam("orgCodeList", orgCodeList)
                ;
        List<TAgentTask> dbTask = tAgentTaskMapper.selectTAgentTaskList(cond);

        Map<String, TaskDistrib.TaskDistribItem> map = ContainerUtils.toMap(task.getItems(), TaskDistrib.TaskDistribItem::getOrgCode);
        Date now = DateUtils.getNowDate();

        dbTask.forEach((x) -> {
            TaskDistrib.TaskDistribItem taskDistribItem = map.get(x.getOrgCode());
            DateTime dateTime = DateUtil.offsetDay(now, taskDistribItem.getDays());
            x.setAgentStatus(TAgentTask.AGENT_STATUS_PROCESSING)
                    .setDistriDate(DateUtils.getNowDate())
                    .setDistriUser(taskDistribItem.getHandleUser())
                    .setDistriNick(sysUserMapper.selectUserByUserName(taskDistribItem.getHandleUser()).getNickName())
                    .setHandleUser(taskDistribItem.getHandleUser())
                    .setEndAt(DateUtil.format(dateTime, "yyyy-MM-dd"))
            ;
        });

        return updateTAgentTaskBatch(dbTask);
    }

    @Override
    public int revoke(TaskRevoke taskRevoke)
    {
        TAgentTask task = taskRevoke.toAgentTask();
        return tAgentTaskMapper.revokeAgentTaskByOrgCode(task);
    }

    @Override
    public int audit(TaskAudit task)
    {
        TAgentTask cond = new TAgentTask();
        cond.setAgentCenter(task.getAgentCenter());
        cond.putParam("idList", task.getTaskIds());
        List<TAgentTask> dbTasks = tAgentTaskMapper.selectTAgentTaskList(cond);

        if(CollectionUtil.isEmpty(dbTasks))
            return 1;

        ASSERT.EXP(dbTasks.stream().allMatch((x) -> TAgentTask.AGENT_STATUS_FINISHED.equals(x.getAgentStatus())), "存在不是已完成的任务");

        dbTasks.forEach((x) -> {
            x.setAgentStatus(TAgentTask.AGENT_STATUS_AUDITED)
                    .setAuditDate(DateUtils.getNowDate())
                    .setAuditUser(task.getAuditUser())
                    .setAuditNick(sysUserMapper.selectUserByUserName(task.getAuditUser()).getNickName())
            ;
        });

        return tAgentTaskMapper.updateTAgentTaskBatch(dbTasks);
    }

    @Override
    public int auditAll(TaskAllAudit taskAllAudit)
    {
        TAgentTask task = taskAllAudit.toAgentTask();
        task.setAuditDate(DateUtils.getNowDate());
        task.setAuditNick(sysUserMapper.selectUserByUserName(task.getAuditUser()).getNickName());
        return tAgentTaskMapper.auditAgentTask(task);
    }

    @Override
    public int finish(TaskFinish task)
    {
        TAgentTask dbTask = tAgentTaskMapper.selectTAgentTaskById(task.getTaskId());

        ASSERT.EXP(TAgentTask.AGENT_STATUS_PROCESSING.equals(dbTask.getAgentStatus()), "任务不是正在处理");
        ASSERT.EXP(task.getHandleUser().equals(dbTask.getHandleUser()), "无权完成此任务");

        dbTask.setHandleRemark(task.getHandleRemark())
                .setAgentStatus(TAgentTask.AGENT_STATUS_FINISHED)
                .setHandleDate(DateUtils.getNowDate())
                .setAuditNick(sysUserMapper.selectUserByUserName(task.getHandleUser()).getNickName())
                ;

        return tAgentTaskMapper.updateTAgentTask(dbTask);
    }

    @Override
    public List<AgentTaskUser> userList(AgentTaskUser agentTaskUser)
    {
        SysUser user = new SysUser();
        user.setAgentCenter(agentTaskUser.getAgentCenter());
        Long deptId = sysDeptMapper.getDeptIdByOrgCode(agentTaskUser.getOrgCode());
        user.setDeptId(deptId);

        List<SysUser> users = sysUserMapper.selectUserList(user);

        if(CollectionUtil.isNotEmpty(users))
        {
            TAgentTask taskCond = new TAgentTask();
            taskCond.putParam("handleUserList", ContainerUtils.mapToList(users, SysUser::getUserName));
            List<TAgentTask> taskHandleUserListGroup = tAgentTaskMapper.getTaskHandleUserListGroup(taskCond);
            Map<String, Long> map = ContainerUtils.toMap(taskHandleUserListGroup, TAgentTask::getHandleUser, TAgentTask::getId);
            Page<AgentTaskUser> tAgentTasks = new Page<>();
            users.forEach((x) -> {
                AgentTaskUser u = new AgentTaskUser();
                u.setNum(map.getOrDefault(x.getUserName(), 0L))
                        .setUserName(x.getUserName())
                        .setNickName(x.getNickName())
                        .setUserId(x.getUserId())
                ;
                tAgentTasks.add(u);
            });
            tAgentTasks.setTotal(new PageInfo(users).getTotal());
            return tAgentTasks;
        }
        else
            return new ArrayList<>();
    }

    @Override
    public List<AgentTaskNumGroup> getUndistribTaskList(TAgentTask tAgentTask, AgentTaskNumGroup summary)
    {
        List<AgentTaskNumGroup> agentTaskNumTownGroup = tAgentTaskMapper.getAgentTaskNumTownGroup(tAgentTask);

        summary.setNumVillage(0L)
                .setNumBook(0L)
                .setAllCount(0L)
                ;

        if(CollectionUtil.isNotEmpty(agentTaskNumTownGroup))
        {
            summary.setNumBook(agentTaskNumTownGroup.stream().map(AgentTaskNumGroup::getNumBook).reduce(0L, Long::sum));
            summary.setNumVillage(agentTaskNumTownGroup.stream().map(AgentTaskNumGroup::getNumVillage).reduce(0L, Long::sum));
            summary.setAllCount(agentTaskNumTownGroup.stream().map(AgentTaskNumGroup::getAllCount).reduce(0L, Long::sum));

            List<AgentTaskNumGroup> agentTaskNumVillageGroup = tAgentTaskMapper.getAgentTaskNumVillageGroup(tAgentTask);
            Map<String, List<AgentTaskNumGroup>> group = ContainerUtils.groupingBy(agentTaskNumVillageGroup, AgentTaskNumGroup::getTownCode);

            agentTaskNumTownGroup.forEach((x) -> {
                x.setOtherCount(x.getAllCount() - x.getAssetCount() - x.getContracCount() - x.getVoucherCount());
                x.setVillageList(group.getOrDefault(x.getTownCode(), new ArrayList<>()));
                x.getVillageList().forEach((y) -> {
                    y.setOtherCount(y.getAllCount() - y.getAssetCount() - y.getContracCount() - y.getVoucherCount());
                });
            });
        }

        return agentTaskNumTownGroup;
    }

    @Override
    public AgentTaskSummary countyTaskSummary(TAgentTask task)
    {
        AgentTaskSummary res = new AgentTaskSummary();

        List<AgentTaskTownGroup> agentTaskNumTownGroup = tAgentTaskMapper.getAgentTaskGroupByTown(task);
        res.setNumTown((long)agentTaskNumTownGroup.size());
        res.setNumBook(agentTaskNumTownGroup.stream().map(AgentTaskTownGroup::getNum).reduce(0L, Long::sum));
        res.setNumTownProcessedTask(agentTaskNumTownGroup.stream().map(AgentTaskTownGroup::getNumProcessFinish).reduce(0L, Long::sum));
        res.setNumTownApprovalTask(agentTaskNumTownGroup.stream().map(AgentTaskTownGroup::getNumApprovalFinish).reduce(0L, Long::sum));

        AgentTaskVillageGroup agentTaskVillageSummary = tAgentTaskMapper.getAgentTaskVillageSummary(task);
        res.setNumBookProcessedTask(agentTaskVillageSummary.getNumProcessFinish())
                .setNumBookApprovalTask(agentTaskVillageSummary.getNumApprovalFinish())
                .setAbnormalCount(agentTaskVillageSummary.getNumExcept())
        ;

        return res;
    }

    @Override
    public List<AgentTaskTownGroup> getProcessingTaskList(TAgentTask tAgentTask)
    {
        tAgentTask.setAgentStatus(TAgentTask.AGENT_STATUS_PROCESSING);
        List<AgentTaskTownGroup> agentTaskGroupByTown = tAgentTaskMapper.getAgentTaskGroupByTown(tAgentTask);
        if(CollectionUtil.isNotEmpty(agentTaskGroupByTown))
        {
            List<String> townCodeList = ContainerUtils.mapToList(agentTaskGroupByTown, AgentTaskTownGroup::getTownCode);

            TAgentTask villageTaskCond = new TAgentTask();
            villageTaskCond.setAgentStatus(TAgentTask.AGENT_STATUS_PROCESSING);
            villageTaskCond.putParam("townCodeList", townCodeList);
            List<AgentTaskVillageGroup> villageGroups = tAgentTaskMapper.getAgentTaskGroupByVillage(villageTaskCond);
            Map<String, List<AgentTaskVillageGroup>> villageMap = ContainerUtils.groupingBy(villageGroups, AgentTaskVillageGroup::getTownCode);

            List<String> orgCodeList = ContainerUtils.mapToList(villageGroups, AgentTaskVillageGroup::getOrgCode);

            TAgentTask agentTaskCond = new TAgentTask();
            agentTaskCond.setAgentStatus(TAgentTask.AGENT_STATUS_PROCESSING);
            agentTaskCond.putParam("orgCodeList", orgCodeList);
            List<TAgentTask> tAgentTasks = tAgentTaskMapper.selectTAgentTaskList(agentTaskCond);
            Map<String, List<TAgentTask>> bookGroup = ContainerUtils.groupingBy(tAgentTasks, TAgentTask::getOrgCode);

            agentTaskGroupByTown.forEach((town) -> {
                List<AgentTaskVillageGroup> villageList = villageMap.getOrDefault(town.getTownCode(), new ArrayList<>());
                town.setVillageList(villageList);
                villageList.forEach((village) -> {
                    village.setDistriDate(DateUtils.stdRemoveTime(village.getDistriDate()));
                    village.setApprovalFinish(village.getNum().equals(village.getNumApprovalFinish()));
                    village.setTaskList(bookGroup.getOrDefault(village.getOrgCode(), new ArrayList<>()));
                    village.getTaskList().forEach((x) -> {
                        x.setOtherCount(x.getAllCount() - x.getVoucherCount() - x.getContracCount());
                    });
                });
            });
        }
        return agentTaskGroupByTown;
    }

    @Override
    public AgentTaskSummary countyProcessingTaskSummary(TAgentTask task)
    {
        AgentTaskSummary res = new AgentTaskSummary();

        task.setAgentStatus(TAgentTask.AGENT_STATUS_PROCESSING);
        List<AgentTaskTownGroup> agentTaskNumTownGroup = tAgentTaskMapper.getAgentTaskGroupByTown(task);
        res.setNumTown((long)agentTaskNumTownGroup.size());
        res.setNumBook(agentTaskNumTownGroup.stream().map(AgentTaskTownGroup::getNum).reduce(0L, Long::sum));
        res.setNumVillage(agentTaskNumTownGroup.stream().map(AgentTaskTownGroup::getNumVillage).reduce(0L, Long::sum));

        AgentTaskVillageGroup agentTaskVillageSummary = tAgentTaskMapper.getAgentTaskVillageSummary(task);
        res.setAbnormalCount(agentTaskVillageSummary.getNumExcept());

        return res;
    }

    @Override
    public List<AgentTaskNumGroup> getAuditingTaskList(TAgentTask tAgentTask)
    {
        tAgentTask.setAgentStatus(TAgentTask.AGENT_STATUS_FINISHED);
        List<AgentTaskNumGroup> villageGroups = tAgentTaskMapper.getAgentTaskNumVillageGroup(tAgentTask);

        if(CollectionUtil.isNotEmpty(villageGroups))
        {
            List<String> orgCodeList = ContainerUtils.mapToList(villageGroups, AgentTaskNumGroup::getOrgCode);

            TAgentTask agentTaskCond = new TAgentTask();
            agentTaskCond.setAgentStatus(TAgentTask.AGENT_STATUS_FINISHED);
            agentTaskCond.putParam("orgCodeList", orgCodeList);
            List<TAgentTask> tAgentTasks = tAgentTaskMapper.selectTAgentTaskList(agentTaskCond);
            Map<String, List<TAgentTask>> bookGroup = ContainerUtils.groupingBy(tAgentTasks, TAgentTask::getOrgCode);

            villageGroups.forEach((village) -> {
                village.setHandleDate(DateUtils.stdRemoveTime(village.getHandleDate()));
                village.setDistriDate(DateUtils.stdRemoveTime(village.getDistriDate()));
                village.setupOverdueFinish();
                village.setOtherCount(village.getAllCount() - village.getContracCount() - village.getVoucherCount());
                village.setTaskList(bookGroup.getOrDefault(village.getOrgCode(), new ArrayList<>()));
                village.getTaskList().forEach((x) -> {
                    x.setOtherCount(x.getAllCount() - x.getVoucherCount() - x.getContracCount());
                });
            });
        }
        return villageGroups;
    }

    @Override
    public AgentTaskSummary auditTaskSummary(TAgentTask task)
    {
        AgentTaskSummary res = new AgentTaskSummary();

        task.setAgentStatus(TAgentTask.AGENT_STATUS_FINISHED);
        AgentTaskTownGroup summary = tAgentTaskMapper.getAgentTaskTownSummary(task);
        res.setNumVillage(summary.getNumVillage())
                .setNumBook(summary.getNum())
                ;

        return res;
    }
}
