package com.ruoyi.common.config;

import com.ruoyi.common.utils.SecretUtils;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.stereotype.Component;

import java.nio.charset.StandardCharsets;

/**
 * 读取项目相关配置
 * 
 * @author ruoyi
 */
@Component
@ConfigurationProperties(prefix = "ruoyi")
public class RuoYiConfig
{
    /** 项目名称 */
    private String name;

    /** 版本 */
    private String version;

    /** 版权年份 */
    private String copyrightYear;

    /** 实例演示开关 */
    private boolean demoEnabled;

    /** 上传路径 */
    private static String profile;

    /** 获取地址开关 */
    private static boolean addressEnabled;

    /** 验证码类型 */
    private static String captchaType;

    public String getName()
    {
        return name;
    }

    public void setName(String name)
    {
        this.name = name;
    }

    public String getVersion()
    {
        return version;
    }

    public void setVersion(String version)
    {
        this.version = version;
    }

    public String getCopyrightYear()
    {
        return copyrightYear;
    }

    public void setCopyrightYear(String copyrightYear)
    {
        this.copyrightYear = copyrightYear;
    }

    public boolean isDemoEnabled()
    {
        return demoEnabled;
    }

    public void setDemoEnabled(boolean demoEnabled)
    {
        this.demoEnabled = demoEnabled;
    }

    public static String getProfile()
    {
        return profile;
    }

    public void setProfile(String profile)
    {
        RuoYiConfig.profile = profile;
    }

    public static boolean isAddressEnabled()
    {
        return addressEnabled;
    }

    public void setAddressEnabled(boolean addressEnabled)
    {
        RuoYiConfig.addressEnabled = addressEnabled;
    }

    public static String getCaptchaType() {
        return captchaType;
    }

    public void setCaptchaType(String captchaType) {
        RuoYiConfig.captchaType = captchaType;
    }

    /**
     * 获取导入上传路径
     */
    public static String getImportPath()
    {
        return getProfile() + "/import";
    }

    /**
     * 获取头像上传路径
     */
    public static String getAvatarPath()
    {
        return getProfile() + "/avatar";
    }

    /**
     * 获取下载路径
     */
    public static String getDownloadPath()
    {
        return getProfile() + "/download/";
    }

    /**
     * 获取上传路径
     */
    public static String getUploadPath()
    {
        return getProfile() + "/upload";
    }


    @Component
    @ConfigurationProperties(prefix = "agentcenter")
    public static class AgentCenter {
        public static Boolean disabled;
        public static String publicKey;
        public static String privateKey;

        public void setDisabled(Boolean disabled) {
            AgentCenter.disabled = disabled;
        }

        public void setPublicKey(String publicKey) {
            AgentCenter.publicKey = publicKey;
        }

        public void setPrivateKey(String privateKey) {
            AgentCenter.privateKey = privateKey;
        }

        public static boolean isDisabled() {
            return null != AgentCenter.disabled /*默认加密*/ && AgentCenter.disabled;
        }
    }
    @Component
    @ConfigurationProperties(prefix = "secret")
    public static class Secret {
        public static Boolean enabled;
        public static String publicKey;
        public static String privateKey;
        public static Boolean compat;

        public void setCompat(Boolean compat) {
            Secret.compat = compat;
        }

        public void setEnabled(Boolean enabled) {
            Secret.enabled = enabled;
        }

        public void setPublicKey(String publicKey) {
            Secret.publicKey = publicKey;
        }

        public void setPrivateKey(String privateKey) {
            Secret.privateKey = privateKey;
        }

        public static boolean isDisabled() {
            return null == Secret.enabled /*默认不加密*/ || !Secret.enabled;
        }

        public static boolean isCompat() {
            return null != Secret.compat /*默认不兼容*/ && Secret.compat;
        }

        // 私钥解密
        public static String decrypt(String inStr) {
            try {
                return SecretUtils.decrypt(inStr, Secret.privateKey, StandardCharsets.UTF_8.name());
            } catch (Exception e) {
                //e.printStackTrace();
                throw new RuntimeException(e);
            }
        }
    }
}
