package com.ruoyi.common.utils;

import java.util.stream.Stream;

public final class ARG<T>
{
    private T[] args;
    public ARG(T...args)
    {
        this.args = args;
    }

    public int argc()
    {
        return Argc((Object[]) args);
    }

    @SuppressWarnings("unchecked")
    public T argv(int i, T...defValue)
    {
        return (T)Argv(i, (T)First((Object[]) defValue));
    }

    public Stream<T> stream(T...args)
    {
        return Stream.of(args);
    }

    public static int Argc(Object...args)
    {
        return null != args ? args.length : 0;
    }

    public static Object Argv(int i, Object...args)
    {
        return Argv_def(i, null, args);
    }

    public static Object Argv_def(int i, Object defValue, Object...args)
    {
        int argc = Argc(args);
        int index = i < 0 ? argc + i : i;
        if(index >= argc)
            throw new RuntimeException(String.format("Argv: index(%d) >= argc(%d)", index, argc));
        Object ret = args[index];
        return null != ret ? ret : defValue;
    }

    public static Object First(Object...args)
    {
        return Argv(0, args);
    }

    public static Object First_def(Object defValue, Object...args)
    {
        return Argv_def(0, null, args);
    }

    public static Stream<Object> Stream(Object...args)
    {
        return Stream.of(args);
    }

    public static <T> T ArgvT_def(int i, T defValue, T...args)
    {
        int argc = Argc((Object[]) args);
        int index = i < 0 ? argc + i : i;
        if(index >= argc)
            throw new RuntimeException(String.format("Argv: index(%d) >= argc(%d)", index, argc));
        T ret = args[index];
        return null != ret ? ret : defValue;
    }

    public static <T> T FirstT(T...args)
    {
        return ArgvT_def(0, null, args);
    }

    public static <T> T FirstT_def(T defValue, T...args)
    {
        return ArgvT_def(0, defValue, args);
    }

    public static <T> Stream<T> StreamT(T...args)
    {
        return Stream.of(args);
    }
}
