package com.ruoyi.common.utils;

import cn.hutool.core.collection.CollectionUtil;
import com.ruoyi.common.core.domain.entity.SysDept;

import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collector;
import java.util.stream.Collectors;

public final class DeptUtils
{
    public static final int LEVEL_INVALID = 0; // 无效
    public static final int LEVEL_PROVINCE = 1; // 省
    public static final int LEVEL_CITY = 2; // 市
    public static final int LEVEL_COUNTY = 3; // 区
    public static final int LEVEL_TOWN = 4; // 镇
    public static final int LEVEL_VILLAGE = 5; // 村
    public static final int LEVEL_GROUP = 6; // 组

    public static String[] parseOrgCode(String orgCode)
    {
        String[] res = new String[7]; // 0-6(0无效), 1省, 2市, 3区, 4镇, 5村, 6组

        res[0] = levelStr(LEVEL_INVALID);
        if(StringUtils.isEmpty(orgCode))
            return res;

        int length = orgCode.length();
        if(length >= 2)
        {
            res[1] = orgCode.substring(0, 2);
            res[0] = levelStr(LEVEL_PROVINCE);
        }
        if(length >= 4)
        {
            res[2] = orgCode.substring(2, 2);
            res[0] = levelStr(LEVEL_CITY);
        }
        if(length >= 6)
        {
            res[3] = orgCode.substring(4, 6);
            res[0] = levelStr(LEVEL_COUNTY);
        }
        if(length >= 9)
        {
            res[4] = orgCode.substring(6, 9);
            res[0] = levelStr(LEVEL_TOWN);
        }
        if(length >= 12)
        {
            res[5] = orgCode.substring(9, 12);
            res[0] = levelStr(LEVEL_VILLAGE);
        }
        if(length >= 16)
        {
            res[6] = orgCode.substring(12, 16);
            res[0] = levelStr(LEVEL_GROUP);
        }

        return res;
    }

    public static String[] parseFullOrgCode(String orgCode)
    {
        String[] res = new String[7]; // 0-6(0无效), 1省, 2市, 3区, 4镇, 5村, 6组

        res[0] = levelStr(LEVEL_INVALID);
        if(StringUtils.isEmpty(orgCode))
            return res;

        int length = orgCode.length();
        if(length >= 2)
        {
            res[1] = orgCode.substring(0, 2);
            res[0] = levelStr(LEVEL_PROVINCE);
        }
        if(length >= 4)
        {
            res[2] = orgCode.substring(0, 2);
            res[0] = levelStr(LEVEL_CITY);
        }
        if(length >= 6)
        {
            res[3] = orgCode.substring(0, 6);
            res[0] = levelStr(LEVEL_COUNTY);
        }
        if(length >= 9)
        {
            res[4] = orgCode.substring(0, 9);
            res[0] = levelStr(LEVEL_TOWN);
        }
        if(length >= 12)
        {
            res[5] = orgCode.substring(0, 12);
            res[0] = levelStr(LEVEL_VILLAGE);
        }
        if(length >= 16)
        {
            res[6] = orgCode.substring(0, 16);
            res[0] = levelStr(LEVEL_GROUP);
        }

        return res;
    }

    public static int parseOrgCodeLevel(String orgCode)
    {
        return StringUtils.isEmpty(orgCode) ? LEVEL_INVALID : parseOrgCodeLevel(orgCode.length());
    }


    public static int parseOrgCodeLevel(int length)
    {
        int res = LEVEL_INVALID;

        if(length >= 16)
        {
            res = LEVEL_GROUP;
        }
        else if(length >= 12)
        {
            res = LEVEL_VILLAGE;
        }
        else if(length >= 9)
        {
            res = LEVEL_TOWN;
        }
        else if(length >= 6)
        {
            res = LEVEL_COUNTY;
        }
        else if(length >= 4)
        {
            res = LEVEL_CITY;
        }
        else if(length >= 2)
        {
            res = LEVEL_PROVINCE;
        }

        return res;
    }

    public static String levelStr(int level)
    {
        return "" + level;
    }

    public static int levelInt(String level)
    {
        return null == level ? LEVEL_INVALID : Integer.parseInt(level);
    }

    public static List<SysDept> getSubDepts_selfAndAllChildren(Long deptId, List<SysDept> depts)
    {
        if(CollectionUtil.isEmpty(depts))
            return new ArrayList<>();
        if(depts.size() == 1)
        {
            SysDept dept = depts.get(0);
            if(deptId.equals(dept.getDeptId()))
            {
                dept.setChildren(new ArrayList<>(depts));
                dept.setChildrenIdList(new ArrayList<>(Collections.singletonList(dept.getDeptId())));
                return depts;
            }
            else
                return new ArrayList<>();
        }

        //depts.removeIf((x) -> deptId.equals(x.getDeptId()));
        depts.forEach((x) -> x.setAncestorsIdList(x.makeAncestorsIdList(true)));
        List<SysDept> res = depts.stream()
                .filter((x) -> deptId.equals(x.getParentId()))
                .peek((x) -> x.setChildren(new ArrayList<>(Collections.singletonList(x))))
                .peek((x) -> x.setChildrenIdList(new ArrayList<>(Collections.singletonList(x.getDeptId()))))
                .peek((x) -> x.getChildren().addAll(depts.stream().filter((y) -> CollectionUtil.contains(y.getAncestorsIdList(), x.getDeptId())).collect(Collectors.toList())))
                .peek((x) -> x.getChildrenIdList().addAll(depts.stream().filter((y) -> CollectionUtil.contains(y.getAncestorsIdList(), x.getDeptId())).map(SysDept::getDeptId).collect(Collectors.toList())))
                .collect(Collectors.toList());
        return res;
    }

    public static <T> Map<Long, List<T>> groupByDept(Long deptId, List<SysDept> allDepts, List<T> list, Function<T, Long> getDeptId, List<SysDept> returnDeptList)
    {
        return groupByDept(deptId, allDepts, list, getDeptId, Collectors.toList(), returnDeptList);
    }

    public static <T, A, R> Map<Long, R> groupByDept(Long deptId, List<SysDept> allDepts, List<T> list, Function<T, Long> getDeptId, Collector<T, A, R> collector, List<SysDept> returnDeptList)
    {
        List<SysDept> depts = getSubDepts_selfAndAllChildren(deptId, allDepts);
        Map<Long, Long> deptMap = new HashMap<>();
        depts.forEach((x) -> {
            deptMap.put(x.getDeptId(), x.getDeptId());
            x.getChildrenIdList().forEach((y) -> {
                deptMap.put(y, x.getDeptId());
            });
        });
        Map<Long, R> collect = list.stream()
                .collect(Collectors.groupingBy((x) -> deptMap.getOrDefault(getDeptId.apply(x), 0L), collector));
        if(null != returnDeptList)
        {
            returnDeptList.addAll(depts);
        }
        return collect;
    }

    private DeptUtils() {}
}
