package com.ruoyi.common.utils;

import com.google.common.eventbus.EventBus;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Component;

import javax.annotation.PreDestroy;
import java.util.ArrayList;
import java.util.List;

@Component
public final class EventBusEngine
{
    private static final Logger       logger        = LoggerFactory.getLogger("EventBusEngine" );
    private static       EventBus     _eventBus;
    private static final List<Object> _listenerList = new ArrayList<>();

    @Autowired
    public void setEventBus(@Qualifier( "syncEventBus" ) EventBus eventBus)
    {
        EventBusEngine._eventBus = eventBus;
        logger.info("EventBusEngine::initialization -> " + eventBus);
    }

    public static void Post(Object message)
    {
        _eventBus.post(message);
    }

    public static void Register(Object listener)
    {
        if(null == listener)
            return;
        synchronized(_listenerList) {
            if(_listenerList.contains(listener))
                return;
            _eventBus.register(listener);
            _listenerList.add(listener);
            logger.info("EventBusEngine::Register -> " + listener);
        }
    }

    public static void Unregister(Object listener)
    {
        if(null == listener)
            return;
        synchronized(_listenerList) {
            if(!_listenerList.contains(listener))
                return;
            _eventBus.unregister(listener);
            _listenerList.remove(listener);
            logger.info("EventBusEngine::Unregister -> " + listener);
        }
    }

    @PreDestroy
    public static void UnregisterAll()
    {
        synchronized(_listenerList) {
            logger.info("EventBusEngine::UnregisterAll -> " + _listenerList.size());
            if(!_listenerList.isEmpty())
            {
                _listenerList.forEach(_eventBus::unregister);
                _listenerList.clear();
            }
        }
    }
}
