package com.ruoyi.common.utils.handler;

import cn.hutool.core.util.StrUtil;
import com.ruoyi.common.utils.spring.SpringUtils;

import java.lang.reflect.InvocationTargetException;
import java.lang.reflect.Method;

final class Handler
{
    public final String protocol;
    public final Object handler;
    public final Method method;

    public Handler(String protocol, Object handler, Method method)
    {
        this.protocol = protocol;
        this.handler = handler;
        this.method = method;
    }

    public Handler(String protocol, Object handler, String methodName)
    {
        this.protocol = protocol;
        this.handler = handler;
        try
        {
            method = handler.getClass().getDeclaredMethod(methodName);
            Class<?>[] parameterTypes = method.getParameterTypes();
        }
        catch(Exception e)
        {
            throw new RuntimeException(e);
        }
    }

    public Handler(String protocol, String beanName, String methodName)
    {
        this.protocol = protocol;
        this.handler = SpringUtils.getBean(beanName);
        try
        {
            method = handler.getClass().getDeclaredMethod(methodName);
            Class<?>[] parameterTypes = method.getParameterTypes();
        }
        catch(Exception e)
        {
            throw new RuntimeException(e);
        }
    }

    public Handler(String protocol, Class<?> beanClazz, String methodName)
    {
        this.protocol = protocol;
        this.handler = SpringUtils.getBean(beanClazz);
        try
        {
            method = handler.getClass().getDeclaredMethod(methodName);
        }
        catch(Exception e)
        {
            e.printStackTrace();
            throw new RuntimeException(e);
        }
    }

    public Object Invoke(Object...args)
    {
        Check(args);
        if(!method.isAccessible())
            method.setAccessible(true);
        try
        {
            return method.invoke(handler, args);
        }
        catch(InvocationTargetException e)
        {
            e.printStackTrace();
            throw new RuntimeException(e.getTargetException());
        }
        catch(Exception e)
        {
            e.printStackTrace();
            throw new RuntimeException(e);
        }
    }

    private void Check(Object...args)
    {
        String err = Match(args);
        if(null != err)
            throw new RuntimeException(err);
    }

    public String Match(Object...args)
    {
        int argsLength = null != args ? args.length : 0;
        Class<?>[] parameterTypes = method.getParameterTypes();
        if(argsLength != parameterTypes.length)
            return StrUtil.format("调用处理器函数参数数量不一致: 需要({})|实际({})", parameterTypes.length, argsLength);
        for(int i = 0; i < argsLength; i++)
        {
            Class<?> parameterType = parameterTypes[i];
            if(null == args[i])
            {
                if(parameterType.equals(boolean.class)
                        || parameterType.equals(byte.class)
                        || parameterType.equals(short.class)
                        || parameterType.equals(int.class)
                        || parameterType.equals(long.class)
                        || parameterType.equals(float.class)
                        || parameterType.equals(double.class)
                        || parameterType.equals(char.class)
                )
                    return StrUtil.format("调用处理器函数第{}个参数传参为null, 但实参是基本数据类型", i + 1);
            }
            else
            {
                if(!parameterType.isAssignableFrom(args[i].getClass()))
                    return StrUtil.format("调用处理器函数第{}个参数传参与实参类型不一致: 需要({})|实际({})", i + 1, parameterType.getName(), args[i].getClass().getName());
            }
        }
        return null;
    }

    @Override
    public String toString()
    {
        return "Handler{" +
                "protocol=" + protocol +
                ", handler=" + handler +
                ", method=" + method +
                '}';
    }
}
