package com.ruoyi.common.utils.sql;

import com.ruoyi.common.exception.UtilException;
import com.ruoyi.common.utils.StringUtils;
import com.ruoyi.common.utils.spring.SpringUtils;
import org.springframework.transaction.PlatformTransactionManager;
import org.springframework.transaction.TransactionDefinition;
import org.springframework.transaction.TransactionStatus;

import java.util.function.Supplier;

/**
 * sql操作工具类
 * 
 * @author ruoyi
 */
public class SqlUtil
{
    /**
     * 定义常用的 sql关键字
     */
    public static String SQL_REGEX = "and |extractvalue|updatexml|exec |insert |select |delete |update |drop |count |chr |mid |master |truncate |char |declare |or |+|user()";

    /**
     * 仅支持字母、数字、下划线、空格、逗号、小数点（支持多个字段排序）
     */
    public static String SQL_PATTERN = "[a-zA-Z0-9_\\ \\,\\.]+";

    /**
     * 检查字符，防止注入绕过
     */
    public static String escapeOrderBySql(String value)
    {
        if (StringUtils.isNotEmpty(value) && !isValidOrderBySql(value))
        {
            throw new UtilException("参数不符合规范，不能进行查询");
        }
        return value;
    }

    /**
     * 验证 order by 语法是否符合规范
     */
    public static boolean isValidOrderBySql(String value)
    {
        return value.matches(SQL_PATTERN);
    }

    /**
     * SQL关键字检查
     */
    public static void filterKeyword(String value)
    {
        if (StringUtils.isEmpty(value))
        {
            return;
        }
        String[] sqlKeywords = StringUtils.split(SQL_REGEX, "\\|");
        for (String sqlKeyword : sqlKeywords)
        {
            if (StringUtils.indexOfIgnoreCase(value, sqlKeyword) > -1)
            {
                throw new UtilException("参数存在SQL注入风险");
            }
        }
    }

    public static void transaction(Runnable runnable)
    {
        PlatformTransactionManager manager = SpringUtils.getBean(PlatformTransactionManager.class);
        TransactionDefinition definition = SpringUtils.getBean(TransactionDefinition.class);
        TransactionStatus transaction = manager.getTransaction(definition);
        try
        {
            runnable.run();
            manager.commit(transaction);
        }
        catch(Exception e)
        {
            e.printStackTrace();
            manager.rollback(transaction);
            throw new RuntimeException(e);
        }
    }

    public static <T> T transaction(Supplier<T> runnable, T...ifExceptRet)
    {
        PlatformTransactionManager manager = SpringUtils.getBean(PlatformTransactionManager.class);
        TransactionDefinition definition = SpringUtils.getBean(TransactionDefinition.class);
        TransactionStatus transaction = manager.getTransaction(definition);
        try
        {
            T res = runnable.get();
            manager.commit(transaction);
            return res;
        }
        catch(Exception e)
        {
            e.printStackTrace();
            manager.rollback(transaction);
            if(null != ifExceptRet && ifExceptRet.length > 0)
                return ifExceptRet[0];
            else
                throw new RuntimeException(e);
        }
    }
}
